package net.scp;


import futils.Futil;
import gui.In;
import gui.dialogs.LabeledItemPanel;
import gui.dialogs.ModalJDialog;
import gui.run.RunButton;
import gui.run.RunPasswordField;
import gui.run.RunTextField;
import gui.run.RunCheckBox;

import javax.swing.*;
import java.beans.PropertyChangeSupport;
import java.io.File;

/**
 * net.scp.ScpDialog
 */
public class ScpDialog extends ModalJDialog {

    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private ScpBean scpBean ;

 

    private LabeledItemPanel c = new LabeledItemPanel();

    public ScpDialog() {
        init();
    }

    private void init() {
        setTitle("Scp Dialog");
        scpBean = ScpBean.restore();
        c.addItem("host", new RunTextField(scpBean.getHost(),20) {
            public void run() {
                scpBean.setHost(getText());
            }
        });
        final RunTextField localFileTextField = getLocalFileTextField();
        c.addItem("localFile", localFileTextField);
        c.addItem("setLocalFile", new RunButton("browse") {
            public void run() {
                File f = Futil.getReadFile("select local file");
                localFileTextField.setText(f.getAbsolutePath());
                scpBean.setLocalFile(f.getAbsolutePath());

            }
        });
        c.addItem("remoteFile", new RunTextField(scpBean.getRemoteFile(),20) {
            public void run() {
                scpBean.setRemoteFile(getText());
            }
        });
        c.addItem("user", new RunTextField(scpBean.getUser(),20) {
            public void run() {
                scpBean.setUser(getText());
            }
        });
        c.addItem("password", new RunPasswordField(scpBean.getPassword(),20) {
            public void run() {
                scpBean.setPassword(getPasswordString());
            }
        });
        c.addItem("protocol", new protocolCheckbox());
        c.setBorder(BorderFactory.createEtchedBorder());

        setContentPane(c);
    }

    private RunTextField getLocalFileTextField() {
        return new RunTextField(
                scpBean.getLocalFile(),20) {
            public void run() {
                scpBean.setLocalFile(getText());
            }
        };
    }

    /**
     * This method gets the values of the panel entry fields.
     *
     * @return an object containing the Customer data
     */
    public ScpBean getData() {
        return scpBean;
    }
    public void addScpChangeEventListener(ScpModelChangeEventListener l){
        pcs.addPropertyChangeListener(l);
    }
    public void okPressed(){
        if (isValidData()){
            processOk();
        }
        else
            debugData();
    }

    private void processOk() {
        scpBean.save();
        final ScpModelChangeEvent evt = new ScpModelChangeEvent(this,scpBean);
        pcs.firePropertyChange(evt);

        scpBean.upload();
        if (In.getBoolean("transfer done, exit?")) System.exit(0);

    }

    public void debugData() {
        In.message("data not valid! Model="+scpBean);

    }


    /**
     * This method checks that the data entered is valid.
     *
     * @return <code>true</code> if the data is valid, otherwise
     *         <code>false</code>
     */
    protected boolean isValidData() {

        return scpBean.isValidData();
    }

    public static void main(String[] args) {
        ScpDialog dialog = new ScpDialog();
        dialog.pack();
        dialog.show();

        System.exit(0);
    }

    private class protocolCheckbox extends RunCheckBox {
        public protocolCheckbox() {
            super(" scp", scpBean.isScp());
            updateLabel();
        }

        public void run() {
            scpBean.setScp(isSelected());
            updateLabel();
        }

        private void updateLabel() {
            if (isSelected())
                setText(" scp");
            else setText("sftp");
        }
    }
}